SulXan = {
    name = "SulXan",
    version = "1.1",

    varVersion = 1, -- savedVariables version
    uiLocked = true,
    defaultSettings = {}
}

local BUFF_ID = 154737 -- Buff id
local SOUL_ID = 154720 -- Soul Drop id

local ST = SulXan
local NAME = ST.name
local EM = EVENT_MANAGER
local SV

local inCombat = false

local stFragment
local sxSlotted = false
local soulEnd = 0
local soulActive = false
local buffEnd = 0
local buffActive = false

local function Initialize()

    SV = ZO_SavedVars:New("SXSV", ST.varVersion, nil, ST.defaultSettings)

    ST.RestorePosition()

    -- Create UI fragment.
    stFragment = ZO_SimpleSceneFragment:New(SulXanControl)
    stFragment:SetConditional(function()
        return sxSlotted and inCombat or not ST.uiLocked
    end)
    HUD_SCENE:AddFragment(stFragment)
    HUD_UI_SCENE:AddFragment(stFragment)

    -- Update CW duration.
    local function UpdateDuration()
        local buffRemain = buffEnd - GetGameTimeSeconds()
        if buffRemain < 0 then
            buffRemain = 0
        end
        local soulRemain = soulEnd - GetGameTimeSeconds()
        if soulRemain < 0 then
            soulRemain = 0
        end

        SulXanControl_CWDuration:SetText(zo_ceil(buffRemain))
        SulXanControl_MEDuration:SetText(zo_ceil(soulRemain))

        if buffActive then
            SulXanControl_BG:SetColor(0, 0, 1)
        end
        if soulActive then
            if not buffActive then
                SulXanControl_BG:SetColor(1 - (6 - 6 / soulRemain) / 6, 0 + (6 - 6 / soulRemain) / 6, 0)
            end
        else
            if not buffActive then
                SulXanControl_BG:SetColor(1, 0, 0)
            end
        end
    end

    local function OnSoul(_, changeType, _, _, _, _, endTime, stackCount, _, _, _, _, _, _, unitId, abilityId)
        soulEnd = endTime
        UpdateDuration()
        if changeType == EFFECT_RESULT_FADED then
            soulActive = false
        elseif (changeType == EFFECT_RESULT_GAINED or changeType == EFFECT_RESULT_UPDATED) then
            soulActive = true
        end
    end

    local function OnBuff(_, changeType, _, _, _, _, endTime, stackCount, _, _, _, _, _, _, unitId, abilityId)
        buffEnd = endTime
        UpdateDuration()
        if changeType == EFFECT_RESULT_FADED then
            buffActive = false
        elseif changeType == EFFECT_RESULT_GAINED or changeType == EFFECT_RESULT_UPDATED then
            buffActive = true
        end
    end

    -- Combat state changes.
    local function CombatState()
        inCombat = IsUnitInCombat("player")
        EM:UnregisterForUpdate(NAME .. 'Update')
        if inCombat and sxSlotted then
            EM:RegisterForUpdate(NAME .. 'Update', 200, function()
                UpdateDuration()
            end)
        end
        stFragment:Refresh()
    end

    -- Check if CW is slotted.
    local function SetCheck()
        local sxNum = 0
        local psxNum = 0
        _, _, _, sxNum, _, _, psxNum = GetItemLinkSetInfo(
            "|H1:item:174533:364:50:26848:370:50:26:0:0:0:0:0:0:0:2049:122:0:1:0:348:0|h|h", true)
        if (sxNum + psxNum) >= 3 then
            sxSlotted = true
        else
            sxSlotted = false
        end
        CombatState()
    end

    EM:RegisterForEvent(NAME, EVENT_PLAYER_ACTIVATED, SetCheck)
    EM:RegisterForEvent(NAME, EVENT_PLAYER_COMBAT_STATE, CombatState)
    EM:RegisterForEvent(NAME, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, SetCheck)

    EM:RegisterForEvent(NAME .. "S", EVENT_EFFECT_CHANGED, OnSoul)
    EM:AddFilterForEvent(NAME .. "S", EVENT_EFFECT_CHANGED, REGISTER_FILTER_ABILITY_ID, SOUL_ID)

    EM:RegisterForEvent(NAME .. "B", EVENT_EFFECT_CHANGED, OnBuff)
    EM:AddFilterForEvent(NAME .. "B", EVENT_EFFECT_CHANGED, REGISTER_FILTER_ABILITY_ID, BUFF_ID)
    EM:AddFilterForEvent(NAME .. "B", EVENT_EFFECT_CHANGED, REGISTER_FILTER_SOURCE_COMBAT_UNIT_TYPE,
        COMBAT_UNIT_TYPE_PLAYER)

end

function ST.Move()

    SV.controlCenterX, SV.controlCenterY = SulXanControl:GetCenter()

    SulXanControl:ClearAnchors()
    SulXanControl:SetAnchor(CENTER, GuiRoot, TOPLEFT, SV.controlCenterX, SV.controlCenterY)

end

function ST.RestorePosition()

    local controlCenterX = SV.controlCenterX
    local controlCenterY = SV.controlCenterY

    if controlCenterX or controlCenterY then
        SulXanControl:ClearAnchors()
        SulXanControl:SetAnchor(CENTER, GuiRoot, TOPLEFT, controlCenterX, controlCenterY)
    end

    SulXanControl_Icon:SetTexture(GetAbilityIcon(BUFF_ID))

end

local function OnAddOnLoaded(event, addonName)
    if addonName == NAME then
        EM:UnregisterForEvent(NAME, EVENT_ADD_ON_LOADED)
        Initialize()
    end
end

EM:RegisterForEvent(NAME, EVENT_ADD_ON_LOADED, OnAddOnLoaded)

SLASH_COMMANDS["/SulXan"] = function(str)
    ST.uiLocked = not ST.uiLocked
    stFragment:Refresh()
end
